#!/usr/bin/perl -w
# ---------------------------------------------------------
# alloid.pl (v 0.1)
# Print all the OIDs supported by an adapter.
# This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.
# (c) 2006 J-L Morel jlmorel@cpan.org
# ---------------------------------------------------------
use strict;
use Win32::NetPacket qw/ GetAdapterNames :oid /;

my %OIDTAG;    # for conversion OID_num --> OID_tag
foreach my $tag ( @{ $Win32::NetPacket::EXPORT_TAGS{'oid'} } ) {
  my $hexa = scalar Win32::NetPacket::constant($tag);
  if ( $hexa !~ /Your vendor/ ) {
    $hexa = sprintf "0x%08X", $hexa;
    $OIDTAG{$hexa} = $tag unless $tag =~ /^OID_GEN_CO_/;
  }
}

# select the adapter
my %desc;
my @adpts = GetAdapterNames( \%desc );
@adpts > 0 or die "No adapter installed !\n";
my $i = 1;
if ( @adpts > 1 ) {
  print "Adapters installed:\n\n";
  print $i++, " - $desc{$_}\n    $_\n" foreach @adpts;
  do {
    print "\nSelect the number of the adapter to open : ";
    $i = <STDIN>;
    chomp $i;
  } until ( $i =~ /^(\d)+$/ and 0 < $i and $i <= @adpts );
}

# open the selected adapter
my $nic = Win32::NetPacket->new( adapter_name => $adpts[ $i - 1 ], );

# retrieve the OID supported list
my $Oid = pack "LLC1024", OID_GEN_SUPPORTED_LIST, 1024;
$nic->GetRequest($Oid) or die "Unable to get OID_GEN_SUPPORTED_LIST";
my ( $code, $num, @supported ) = unpack "LLL*", $Oid;

# print the list
foreach (@supported) {
  last unless $_;
  my $hexa = sprintf "0x%08X", $_;
  printf "$hexa == %s\n", exists $OIDTAG{$hexa} ? $OIDTAG{$hexa} : '??';
}
